@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,11 +27,11 @@ import time
 import errno
 import socket
 import select
-import tempfile
 
 from . import absolute_url, StoringHandler, get_url_from
 from .. import (log, LOG_CHECK, LOG_CACHE, httputil, httplib2 as httplib,
-  strformat, LinkCheckerError, url as urlutil, trace, clamav, winutil, geoip)
+  strformat, LinkCheckerError, url as urlutil, trace, clamav, winutil, geoip,
+  fileutil)
 from ..HtmlParser import htmlsax
 from ..htmlutil import linkparse
 from ..network import iputil
@@ -39,7 +39,8 @@ from .const import (WARN_URL_EFFECTIVE_URL,
     WARN_URL_ERROR_GETTING_CONTENT, WARN_URL_OBFUSCATED_IP,
     WARN_URL_ANCHOR_NOT_FOUND, WARN_URL_WARNREGEX_FOUND,
     WARN_URL_CONTENT_SIZE_TOO_LARGE, WARN_URL_CONTENT_SIZE_ZERO,
-    WARN_URL_CONTENT_SIZE_UNEQUAL, ExcList, ExcSyntaxList, ExcNoCacheList)
+    WARN_URL_CONTENT_SIZE_UNEQUAL, WARN_URL_WHITESPACE,
+    ExcList, ExcSyntaxList, ExcNoCacheList)
 
 # helper alias
 unicode_safe = strformat.unicode_safe
@@ -77,11 +78,16 @@ class UrlBase (object):
     ContentMimetypes = {
         "text/html": "html",
         "application/xhtml+xml": "html",
+        # Include PHP file which helps when checking local .php files.
+        # It does not harm other URL schemes like HTTP since HTTP servers
+        # should not send this content type. They send text/html instead.
+        "application/x-httpd-php": "html",
         "text/css": "css",
         "application/x-shockwave-flash": "swf",
         "application/msword": "word",
         "text/plain+linkchecker": "text",
         "text/plain+opera": "opera",
+        "text/plain+chromium": "chromium",
     }
 
     def __init__ (self, base_url, recursion_level, aggregate,
@@ -100,9 +106,9 @@ class UrlBase (object):
         @param name: name of url or empty
         @param url_encoding: encoding of URL or None
         """
+        self.reset()
         self.init(base_ref, base_url, parent_url, recursion_level,
                   aggregate, line, column, name, url_encoding)
-        self.reset()
         self.check_syntax()
 
     def init (self, base_ref, base_url, parent_url, recursion_level,
@@ -111,8 +117,10 @@ class UrlBase (object):
         Initialize internal data.
         """
         self.base_ref = base_ref
-        # note that self.base_url must not be modified
-        self.base_url = base_url
+        self.base_url = base_url.strip() if base_url else base_url
+        if self.base_url != base_url:
+            self.add_warning(_("Leading or trailing whitespace in URL `%(url)s'.") %
+                               {"url": base_url}, tag=WARN_URL_WHITESPACE)
         self.parent_url = parent_url
         self.recursion_level = recursion_level
         self.aggregate = aggregate
@@ -126,7 +134,7 @@ class UrlBase (object):
         if self.parent_url:
             assert not urlutil.url_needs_quoting(self.parent_url), \
                    "unquoted parent URL %r" % self.parent_url
-        url = absolute_url(base_url, base_ref, parent_url)
+        url = absolute_url(self.base_url, base_ref, parent_url)
         # assume file link if no scheme is found
         self.scheme = url.split(":", 1)[0] or "file"
 
@@ -384,7 +392,7 @@ class UrlBase (object):
         elif self.parent_url:
             # strip the parent url query and anchor
             urlparts = list(urlparse.urlsplit(self.parent_url))
-            urlparts[3] = urlparts[4] = ""
+            urlparts[4] = ""
             parent_url = urlparse.urlunsplit(urlparts)
             self.url = urljoin(parent_url, base_url, self.scheme)
         else:
@@ -451,7 +459,7 @@ class UrlBase (object):
         """Try to ask GeoIP database for country info."""
         if self.host:
             country = geoip.get_country(self.host)
-            if country is not None:
+            if country:
                 self.add_info(_("URL is located in %(country)s.") %
                 {"country": _(country)})
 
@@ -708,6 +716,7 @@ class UrlBase (object):
                 self.scan_virus()
 
     def check_warningregex (self):
+        """Check if content matches a given regular expression."""
         warningregex = self.aggregate.config["warningregex"]
         if warningregex:
             log.debug(LOG_CHECK, "checking content")
@@ -745,12 +754,11 @@ class UrlBase (object):
     def check_html (self):
         """Check HTML syntax of this page (which is supposed to be HTML)
         with the local HTML tidy module."""
-        try:
-            import tidy
-        except ImportError:
+        if not fileutil.has_module("tidy"):
             log.warn(LOG_CHECK, _("warning: tidy module is not available; " \
                  "download from http://utidylib.berlios.de/"))
             return
+        import tidy
         options = dict(output_html=0, show_warnings=1, quiet=True,
             input_encoding='utf8', output_encoding='utf8', tidy_mark=0)
         try:
@@ -919,22 +927,20 @@ class UrlBase (object):
     def parse_opera (self):
         """Parse an opera bookmark file."""
         log.debug(LOG_CHECK, "Parsing Opera bookmarks %s", self)
-        name = None
-        lineno = 0
-        for line in self.get_content().splitlines():
-            lineno += 1
-            line = line.strip()
-            if line.startswith("NAME="):
-                name = line[5:]
-            elif line.startswith("URL="):
-                url = line[4:]
-                if url and name is not None:
-                    url_data = get_url_from(url, self.recursion_level+1,
-                        self.aggregate, parent_url=self.url,
-                        line=lineno, name=name)
-                    self.aggregate.urlqueue.put(url_data)
-            else:
-                name = None
+        from ..bookmarks.opera import parse_bookmarks
+        for url, name, lineno in parse_bookmarks(self.get_content()):
+            url_data = get_url_from(url, self.recursion_level+1,
+               self.aggregate, parent_url=self.url, line=lineno, name=name)
+            self.aggregate.urlqueue.put(url_data)
+
+    def parse_chromium (self):
+        """Parse a Google Chrome bookmark file."""
+        log.debug(LOG_CHECK, "Parsing Chromium bookmarks %s", self)
+        from ..bookmarks.chromium import parse_bookmarks_data
+        for url, name in parse_bookmarks_data(self.get_content()):
+            url_data = get_url_from(url, self.recursion_level+1,
+               self.aggregate, parent_url=self.url, name=name)
+            self.aggregate.urlqueue.put(url_data)
 
     def parse_text (self):
         """
@@ -1006,12 +1012,12 @@ class UrlBase (object):
 
     def get_temp_filename (self):
         """Get temporary filename for content to parse."""
-        # XXX move to utility package
         # store content in temporary file
-        fd, filename = tempfile.mkstemp(suffix='.doc', prefix='lc_')
-        fp = os.fdopen(fd)
-        fp.write(self.get_content())
-        fp.close()
+        fd, filename = fileutil.get_temp_file(suffix='.doc', prefix='lc_')
+        try:
+            fd.write(self.get_content())
+        finally:
+            fd.close()
 
     def serialized (self):
         """
@@ -1076,7 +1082,7 @@ class UrlBase (object):
           Indicates if URL data has been loaded from cache.
         - url_data.result: unicode
           Result string
-        - url_data.warnings: list of unicode
+        - url_data.warnings: list of tuples (tag, warning message)
           List of tagged warnings for this URL.
         - url_data.name: unicode string or None
           name of URL (eg. filename or link name)
@@ -1104,12 +1110,14 @@ class UrlBase (object):
           Cache key for this URL.
         - url_data.content_type: unicode
           MIME content type for URL content.
+        - url_data.level: int
+          Recursion level until reaching this URL from start URL
         """
         return dict(valid=self.valid,
           extern=self.extern[0],
           cached=self.cached,
           result=self.result,
-          warnings=[x[1] for x in self.warnings],
+          warnings=self.warnings[:],
           name=self.name or u"",
           title=self.get_title(),
           parent_url=self.parent_url or u"",
@@ -1125,9 +1133,12 @@ class UrlBase (object):
           column=self.column,
           cache_url_key=self.cache_url_key,
           content_type=self.get_content_type(),
+          level=self.recursion_level,
         )
 
     def to_wire (self):
+        """Return compact UrlData object with information from to_wire_dict().
+        """
         return CompactUrlData(self.to_wire_dict())
 
 
@@ -1158,9 +1169,11 @@ urlDataAttr = [
     'column',
     'cache_url_key',
     'content_type',
+    'level',
 ]
 
 class CompactUrlData (object):
+    """Store selected UrlData attributes in slots to minimize memory usage."""
     __slots__ = urlDataAttr
 
     def __init__(self, wired_url_data):
