@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2008-2010 Bastian Kleineidam
+# Copyright (C) 2008-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,11 +30,12 @@ from .options import LinkCheckerOptions
 from .checker import CheckerThread
 from .contextmenu import ContextMenu
 from .editor import EditorWindow
+from .updater import UpdateDialog
 from .urlmodel import UrlItemModel
 from .urlsave import urlsave
 from .settings import Settings
 from .. import configuration, checker, director, add_intern_pattern, \
-    strformat, fileutil
+    strformat, fileutil, LinkCheckerError
 from ..containers import enum
 from .. import url as urlutil
 from ..checker import httpheaders
@@ -45,10 +46,30 @@ RegistryBase = "Bastian"
 Status = enum('idle', 'checking')
 
 
+def get_app_style ():
+    """Return appropriate QStyle object for the current platform to
+    be used in QApplication.setStyle().
+    Currently prefers Macintosh on OS X, else Plastique.
+    Note that style names are case insensitive.
+
+    See also
+    http://doc.trolltech.com/latest/gallery-macintosh.html
+    and
+    http://doc.trolltech.com/latest/gallery-plastique.html
+    """
+    if sys.platform == 'darwin':
+        style = "Macintosh"
+    else:
+        style = "Plastique"
+    return QtGui.QStyleFactory.create(style)
+
+
 class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
+    """The main window displaying checked URLs."""
 
     log_url_signal = QtCore.pyqtSignal(object)
     log_stats_signal = QtCore.pyqtSignal(object)
+    error_signal = QtCore.pyqtSignal(str)
 
     def __init__(self, parent=None, url=None):
         """Initialize UI."""
@@ -69,6 +90,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.editor = EditorWindow(parent=self)
         # Note: do not use QT assistant here because of the .exe packaging
         self.assistant = HelpWindow(self, self.get_qhcpath())
+        self.config_error = None
         # init the rest
         self.init_treeview()
         self.connect_widgets()
@@ -76,6 +98,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.init_app()
 
     def init_app (self):
+        """Set window size and position, GUI options and reset status."""
         data = self.settings.read_geometry()
         if data["size"] is not None:
             self.resize(data["size"])
@@ -84,7 +107,8 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.options.set_options(self.settings.read_options())
         self.status = Status.idle
         self.actionSave.setEnabled(False)
-        self.set_statusbar(_("Ready."))
+        msg = self.config_error or _("Ready.")
+        self.set_statusbar(msg)
 
     def get_qhcpath (self):
         """Helper function to search for the QHC help file in different
@@ -108,13 +132,16 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         Autoconnected methods have the form on_<objectname>_<signal>.
         """
         def set_idle ():
+            """Set application status to idle."""
             self.status = Status.idle
         self.checker.finished.connect(set_idle)
         self.checker.terminated.connect(set_idle)
         self.log_url_signal.connect(self.model.log_url)
         self.log_stats_signal.connect(self.log_stats)
+        self.error_signal.connect(self.internal_error)
 
     def init_treeview (self):
+        """Set treeview model and layout."""
         self.model = UrlItemModel()
         self.treeView.setModel(self.model)
         data = self.settings.read_treeviewcols()
@@ -125,6 +152,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         selectionModel.selectionChanged.connect(self.set_properties)
 
     def get_treeviewcols (self):
+        """Return URL treeview column widths."""
         return dict(
             col1=self.treeView.columnWidth(0),
             col2=self.treeView.columnWidth(1),
@@ -134,6 +162,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     def init_config (self):
         """Create a configuration object."""
         self.config = configuration.Configuration()
+        # set standard GUI configuration values
         self.config.logger_add("gui", GuiLogger)
         self.config["logger"] = self.config.logger_new('gui',
             signal=self.log_url_signal, stats=self.log_stats_signal)
@@ -142,6 +171,11 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.handler = GuiLogHandler(self.debug.log_msg_signal)
         status = StatusLogger(self.progress.log_status_signal)
         self.config.init_logging(status, handler=self.handler)
+        # read user and system configuration file
+        try:
+            self.config.read()
+        except LinkCheckerError, msg:
+            self.config_error = msg
 
     def set_config (self):
         """Set configuration."""
@@ -156,9 +190,11 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
             self.config.reset_loglevel()
 
     def get_status (self):
+        """Return current application status."""
         return self._status
 
     def set_status (self, status):
+        """Set application status."""
         self._status = status
         if status == Status.idle:
             self.progress.hide()
@@ -206,22 +242,32 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     @QtCore.pyqtSlot()
     def on_actionAbout_triggered (self):
         """Display about dialog."""
+        modules = u"<br>\n".join(configuration.get_modules_info())
         d = {
             "app": configuration.App,
             "appname": configuration.AppName,
             "copyright": configuration.HtmlCopyright,
+            "donateurl": configuration.DonateUrl,
+            "pyver": u"%d.%d.%d" % sys.version_info[:3],
+            "modules": modules,
         }
         QtGui.QMessageBox.about(self, _(u"About %(appname)s") % d,
-            _(u"""<qt><p>%(appname)s checks HTML documents and websites
-for broken links.
+            _(u"""<qt><center>
+<h1>%(app)s</h1>
+<p>Python: %(pyver)s<br>
+%(modules)s
 <p>%(copyright)s
-<br>%(app)s is licensed under the
+<br>%(appname)s is licensed under the
 <a href="http://www.gnu.org/licenses/gpl.html">GPL</a>
 Version 2 or later.
-<p>Please consider a
-<a href="https://sourceforge.net/donate/index.php?group_id=1913">donation</a>
-to improve %(appname)s even more!
-</qt>""") % d)
+<p>If you like %(appname)s, consider one of several ways to
+<a href="%(donateurl)s">donate</a>. Thanks!
+</center></qt>""") % d)
+
+    @QtCore.pyqtSlot()
+    def on_actionDonate_triggered (self):
+        """Open donation URL."""
+        webbrowser.open(configuration.DonateUrl)
 
     @QtCore.pyqtSlot()
     def on_actionDebug_triggered (self):
@@ -233,6 +279,13 @@ to improve %(appname)s even more!
         """Quit application."""
         urlsave(self, self.config, self.model.urls)
 
+    @QtCore.pyqtSlot()
+    def on_actionCheckUpdates_triggered (self):
+        """Display update check result."""
+        dialog = UpdateDialog(self)
+        dialog.reset()
+        dialog.show()
+
     def start (self):
         """Start a new check."""
         if self.status == Status.idle:
@@ -242,14 +295,14 @@ to improve %(appname)s even more!
 
     def get_url (self):
         """Return URL to check from the urlinput widget."""
-        url = unicode(self.urlinput.text()).strip()
+        url = strformat.stripall(unicode(self.urlinput.text()))
         if url.startswith(u"www."):
             url = u"http://%s" % url
         elif url.startswith(u"ftp."):
             url = u"ftp://%s" % url
         elif url and u":" not in url:
-            # Look for local filename, else assume it's an HTTP URL.
-            if not os.path.isfile(url):
+            # Look for local file, else assume it's an HTTP URL.
+            if not os.path.exists(url):
                 url = u"http://%s" % url
         return url
 
@@ -292,13 +345,6 @@ to improve %(appname)s even more!
             self.contextmenu.enableFromItem(urlitem)
             self.contextmenu.popup(QtGui.QCursor.pos())
 
-    @QtCore.pyqtSlot()
-    def on_actionViewProperties_triggered (self):
-        """View URL data properties in a separate window."""
-        urlitem = self.model.getUrlItem(self.treeView.currentIndex())
-        if urlitem is not None:
-            self.view_item_properties(urlitem)
-
     @QtCore.pyqtSlot()
     def on_actionViewOnline_triggered (self):
         """View item URL online."""
@@ -322,7 +368,9 @@ to improve %(appname)s even more!
                              urlitem.url_data.line, urlitem.url_data.column)
 
     def view_source (self, url, line, col):
+        """View URL source in editor window."""
         self.editor.setWindowTitle(u"View %s" % url)
+        self.editor.setUrl(url)
         info, data = urlutil.get_content(url, proxy=self.config["proxy"])
         if (info, data) == (None, None):
             self.editor.setText(u"An error occurred retreiving URL `%s'." % url)
@@ -351,4 +399,9 @@ to improve %(appname)s even more!
         self.statusBar.showMessage(msg)
 
     def log_stats (self, statistics):
+        """Set statistic information for selected URL."""
         set_statistics(self, statistics)
+
+    def internal_error (self, msg):
+        """Display internal error message. Triggered by sys.excepthook()."""
+        QtGui.QMessageBox.warning(self, _(u"LinkChecker internal error"), msg)
